import mx.central.Central;
import mx.central.Shell;
import mx.central.data.LCService;
import mx.central.data.LCDataProvider;
import mx.central.data.SelectedItem;
import mx.controls.Alert;
import mx.core.UIComponent;
import mx.core.UIObject;
import mx.utils.Delegate;

/**
 * Stock Watcher sample application for Macromedia Central 1.5.
 *
 * This extends the MovieClip class because this instance will be tied to the main 
 * user interface MovieClip on the stage.
 *
 * @author Macromedia
 * @version 1.5
 */
class StockWatcherApp extends MovieClip implements mx.central.Application, IStockClient
{
	/**
	 * Reference to the Central Shell API
	 */
	var oShell:Shell; 
	
	/**
	 * Local connection listener to communicate with pod and agent
	 */
    var oConn;         			
	/**
	 * Unique name of this application.
	 */
	var sAppName:String = "StockWatcher";
	/**
	 * Remote data connection to agent for company data
	 *
	 * @see mx.central.data.LCDataProvider
	 */
    var oCompanies:LCDataProvider; 
	/**
	 * The last time any stock was updated
	 */
    var dLastUpdate:Date;
	/**
	 * The currently selected stock symbol, if any
	 */
    var sCurrentStock:String;
	/**
	 * The previously selected stock symbol.
	 * Used to prevent repeated lookups of the same stock.
	 */
    var sPrevStock:String; 
	
	/**
	 * Content MovieClip for the Alert Rule preferences
	 */
	var fPrefs:MovieClip;
	/**
	 * Whether the preferences dialog box is shown or hidden
	 */
	var bIsPrefsOpen:Boolean = false;
	/**
	 * Whether this class has been initialized or not.
	 * Used to prevent initialization from happening a second time.
	 */
	var bIsInitialized:Boolean = false;
	
	/**
	 * Acts as a border around the details area
	 */
	var fDetailsTab:mx.central.controls.MAccordionTab;
	/**
	 * Acts as a border around the main companies grid area
	 */
	var fFindTab:mx.central.controls.MAccordionTab;
	/**
	 * The button that opens the pod
	 */
	var fTossButton:mx.central.controls.MIconButton;
	/**
	 * The button that opens the preferences dialog
	 */
	var fAlertsButton:mx.central.controls.MIconButton;
	/**
	 * The button that retrieves updated company data
	 */
	var fRefreshButton:mx.controls.Button;
	/**
	 * The button that adds a new company to the list
	 */
	var fAddButton:mx.controls.Button;
	/**
	 * The grid that shows all the company information
	 */
	var fResultsGrid:mx.controls.DataGrid;
	/**
	 * The text field that shows the last updated date for the data
	 */
	var fUpdated:TextField;
	/**
	 * MovieClip containing all of the detail text fields
	 */
	var fDetails:MovieClip;
	/**
	 * Stock symbol input text field.
	 */
	var fSymbol:mx.controls.TextInput;

    static var kDefaultButtonWidth = 60;

	/**
	 * Constructor
	 */
    function StockWatcher() {
		StockUtils.trace("constructor","StockWatcherApp");
    }
	
	/**
	 * Sets up component handlers and does any other things that should happen before Central 
	 * gets notified about this application.
	 */
	public function init():Void {
		StockUtils.trace("init()","StockWatcherApp");
						
		// Stylizes V2 components
		StockStyles.defineStyles();
		this.fAddButton.setStyle("styleName", StockStyles.kStyleName);
		this.fRefreshButton.setStyle("styleName", StockStyles.kStyleName);
		this.fSymbol.setStyle("styleName", StockStyles.kStyleName);
		
		//this.fResultsGrid.setStyle("styleName", StockStyles.kGridStyleName);
		StockStyles.setGridStyles(this.fResultsGrid);
	
		// Sets up V2 Component event handlers
		// Delegate.create() sets up a proxy function that removes some scoping complication.
		this.fAddButton.addEventListener("click", Delegate.create(this, addSymbol));
		
		// Traps the enter key on the symbol field and add the new company
		this.fSymbol.addEventListener("enter", Delegate.create(this, addSymbol));
		
		this.fRefreshButton.addEventListener("click", Delegate.create(this, refreshData));
		
		// When a row is selected in the grid, shows that company's details
		this.fResultsGrid.addEventListener("change", Delegate.create(this, showSelectedCompany));
		
		// Central Button components use different event handling schemes than V2 Buttons
		this.fTossButton.setChangeHandler("openStockPod");
		
		// Alerts button brings up prefs
		this.fAlertsButton.setChangeHandler("showPreferences");
		
		this.fDetails.fRemoveButton.setChangeHandler("removeCompany");
		
		// Remove button is hidden unless a company is selected
		this.fDetails.fRemoveButton._visible = false;
		
		// Tells Central that the application is ready to go.
		// Since this class also represents the main MovieClip on the stage
		Central.initApplication(this, this);
	}
    
	/**
	 * Called by the shell when an application is instantiated (that is, becomes active).
	 *
     * @see mx.central.Application#onActivate
	 * @see mx.central.Central#initApplication
	 * @param shell () Shell object; used to reference functions in the Central Shell API
	 * @param appID (Number) Unique ID for this application, common to all pieces 
	 *                       (application, pods, and agent)
	 * @param shellID (Number) Unique ID for the particular shell instance (Central window) 
	 *                        in which this application is running.
	 * @param baseTabIndex (Number) Sets tab indexes on controls such as buttons and text fields 
	 *                        starting with this number.
	 * @param initialData (Object) Any data type; passed to your application at launch 
	 *                        (primarily from the product.xml's initialData tag).
	 **/
	function onActivate (shell:Shell, appID:Number, shellID:Number,
                         baseTabIndex:Number, initialData:Object):Void
    {
		StockUtils.trace("onActivate()", "StockWatcherApp");
		StockUtils.trace("DataGrid version=" + mx.controls.DataGrid.version);

        // keep some references around
        oShell = shell;
        sAppName = sAppName + shellID;

        // start agent in case it's not running
        oShell.startAgent();

        // Establishes a local connection to the StockService (owned by the agent).
		// The last parameter, bSync, is set to false because a couple of the 
		// methods on the StockService can take a while to execute (especially the
		// one that makes a web service call for each stock in your list).
		// If your method calls are all of short duration, you can set bSync to
		// true and treat all the calls as synchronous.
        oConn = LCService.createClient( StockInterfaces, sAppName, this, false);
		
		// Allows the agent to talk to anybody.
		oConn.allowDomain = function(domain:String):Boolean {
			return true;
		};	
		
        oCompanies = LCDataProvider.createClient( sAppName, "companies" );

		// handle major changes in data
        var companiesListener = new Object;
		companiesListener.owner = this;
        companiesListener.modelChanged = function(eventObj:Object) 
		{
			StockUtils.trace("companiesListener.modelChanged event=" + eventObj.eventName, "StockWatcherApp");
			
			if (eventObj.eventName == "addItems") 
			{
				this.owner.refreshGrid()
			}
        }
        oCompanies.addListener(companiesListener);
	    // drive the results grid off the LCDataProvider
		this.fResultsGrid.dataProvider = oCompanies;
	
        // request the freshness date of the data
        oConn.getLastUpdate();
		
        // do an initial sizing
        onResize();
		
		// sets grid headers after resizing, so column widths are correct
		setGridHeaders(this.fResultsGrid);
    }
	
	/**
	 * Select the current stock in the grid
	 */
	public function refreshGrid() {		
		if (this.sCurrentStock && this.sCurrentStock != this.sPrevStock)
		{
			this.sPrevStock = this.sCurrentStock;
			// first try the most recently selected stock
			this.findCompanyInGrid(this.sCurrentStock);
		}
		else if (this.fSymbol.text)
		{
			// otherwise get the stock most recently searched for
			this.findCompanyInGrid(this.fSymbol.text);
		} 
		else if (this.fResultsGrid.length > 0) 
		{
			// finally select the first line by default
			this.fResultsGrid.selectedIndex = 0;
			this.fResultsGrid.dispatchEvent({type:"change", target:this.fResultsGrid});
		}
	}
		
	/**
	 * Called if the LCService connection fails
	 */
	public function createClient_Status (clientObject:Object) {
		StockUtils.trace("LCService.createClient failed.");
		showError("Unable to connect to the Stock Watcher Agent. Please close this application and try again later.");
	}
	
	/**
	 * Called when the LCService connection to the AIM service is successful
	 */
	function createClient_Result (proxyObject:Object) {
		StockUtils.trace("LCService.createClient succeeded.");
	}
	/**
	 * Called by the shell the instant before the application instance 
	 * is unloaded.
	 **/	
	function onDeactivate (Void):Void
    {
        this.oConn.close();
        this.oShell.stopAgent();
    }
	
	/**
	 * Handles clicks on the Add button
	 */
	function addSymbol(uiEvent:Object) 
	{
		StockUtils.trace("addSymbol()", "StockWatcherApp");
		if (this.fSymbol.text == "") 
		{
			return;
		}
		this.oConn.addCompany(fSymbol.text);
		this.fSymbol.text = "";
	}
	
	/**
	 * Handles clicks on the Remove button
	 */
	function removeCompany(uiEvent:Object) 
	{
		StockUtils.trace("removeCompany()", "StockWatcherApp");
		//remove the currently selected stock from the list
        this.oConn.removeCompany(sCurrentStock);
    }
	
	/**
	 * Displays the details for a company that is selected in the DataGrid
	 */
	function showSelectedCompany(uiEvent:Object) 
	{	            
		
		// find selected symbol -- can't just use index of grid
		// item since rows may have been sorted
		var item:StockItem = this.fResultsGrid.selectedItem;
		StockUtils.trace("showSelectedCompany(" + item.symbol + ")", "StockWatcherApp");
		if (item != undefined) {
			this.sCurrentStock = item.symbol;
		
			showCompany(item);
		}
	}
	/**
	 * Called by the shell when the application window is resized, to trigger custom layout code.
	 **/
	function onResize (Void):Void
    {
		StockUtils.trace("onResize()", "StockWatcherApp");
        // make sure to get bounds, rather than stage width/height
        var bounds:Object = oShell.getBounds();
        StockUtils.trace("bounds=" + bounds.width + "," + bounds.height, "StockWatcherApp");    
        
        // layout panes - grid flexes, company info does not.
        // this means that the company info moves up and down when the
        // grid flexes.
        this.fDetailsTab.setSize(bounds.width - (this.fDetailsTab._x*2), null);
        this.fDetailsTab._y = bounds.height - this.fDetailsTab._height - 5;
		
        this.fFindTab.setSize(bounds.width - (this.fFindTab._x*2),
                         this.fDetailsTab._y - this.fFindTab._y - 10);

        // layout content in top area - move buttons around so that
        // they spread out evenly when the window gets wider/narrower
        var contentbounds:Object = this.fFindTab.getContentBounds();
        StockUtils.trace("contentbounds=" + contentbounds.xMax + "," + contentbounds.yMax, "StockWatcherApp");    
       
	    this.fTossButton._x = contentbounds.xMax - this.fTossButton._width - 5;
        this.fAlertsButton._x = this.fTossButton._x - this.fAlertsButton._width - 3;
        this.fRefreshButton._x = this.fAlertsButton._x - kDefaultButtonWidth - 5;
        this.fUpdated._x = this.fAddButton._x + kDefaultButtonWidth;
        this.fUpdated._width = this.fRefreshButton._x - this.fUpdated._x - 5;
		
        fResultsGrid.setSize(contentbounds.xMax - 32,  contentbounds.yMax - fResultsGrid._y - 5);
	
        // layout details in bottom area
        // just line up fDetails with fDetailsTab
        contentbounds = this.fDetailsTab.getContentBounds();
        this.fDetails._y = this.fDetailsTab._y + 13;
        this.fDetails.fRemoveButton._x = contentbounds.xMax - this.fDetails.fRemoveButton._width - 5;
	
        // reposition preferences dialog
		if (fPrefs != undefined) {
        	fPrefs.resizePreferences(bounds.width, bounds.height);
		}
    }
	
	/**
	 * Invoked when a notice created by this app is dismissed. Triggered when the user 
	 * clicks the close box or the engage text, if the notice times out, or the notice 
	 * is removed using removeNotice().
	 *
	 * @param event (Object) Object containing one string element, type, that provides the reason 
	 *                       for the notice's dismissal (close, engage, timeout, remove)
	 * @param noticeData (Object) Object containing detailed information about the notice.
	 * @param initialData (Object) Any data type specifying application-specific data passed 
	 *                       at the time you call addNotice().
	 **/
	function onNoticeEvent (event:Object, noticeData:Object, initalData:Object):Void
    {
		// not implemented
    }
	
	/**
	 * Called by the shell when the connection status changes.
	 * 
	 * @param connected (Boolean) true if user is connected, false if offline
	 **/        
	function onNetworkChange (connected:Boolean):Void
    {
		// not implemented
    }

	/**
	 * Called by the shell when data is arriving in your application from a Blast operation.
	 *
	 * @tiptext Called by the shell when data is arriving in your application from a Blast operation.
	 * @see mx.central.Application
	 * @param data (Array) An array of instances of the SelectedItem ActionScript structure or 
	 *                     an array of XML objects.
	 **/
	function onSelectedItem (data:Array):Void
    {
        if (data instanceof Array) {

            // loop through a bunch of selected items
            for (var i:Number=0; i<data.length; i++) {
                processSelectedItem(data[i].asSelectedItem());
            }
        } else {
            // process a single selected item
            processSelectedItem(data.asSelectedItem());
        }
    }      
	/**
	 * Called when the application is being uninstalled from Central, to give the app a chance 
	 * to clean up (for example, by clearing local shared objects).
	 */
	function onUninstall (Void):Void {
		oConn.deleteStorage();
	}
	
	/**
	 * Asks for updated data from the server
	 */
	function refreshData(uiEvent:Object):Void {
		StockUtils.trace("refreshData()", "StockWatcherApp");
		
		// Shows status message and progress bar
		this.oShell.setStatus("getting updated data...");
		this.oShell.setProgress(-1);
		
		// tell the Agent to get the latest info
		oConn.think();
	}

	/**
	 * Called when the server has stopped receiving updated data
	 */
	function think_Result(isDone:Boolean):Void {
		StockUtils.trace("think_Result()", "StockWatcherApp");
		// hide status message and progress bar
		this.oShell.setStatus("");
		this.oShell.setProgress(0);
	}
	
	/**
	 * Called by the shell when the user selects the menu option to view their app's preferences.
	 *
	 * @tiptext Called by the shell when the user selects the menu option to view their app's preferences.
	 * @see mx.central.Application
	 **/	
	function showPreferences(Void):Void {
		this.bIsPrefsOpen = true;
		
		if (this.fPrefs == undefined) {
			var bounds = oShell.getBounds();
			var obj = {};
			obj.controller = this;
			obj.width = 420;
			obj.height = 320;
			obj._x = (bounds.width - obj.width)/2;
			obj._y = (bounds.height - obj.height)/2;
			obj.isModal = true;
		
			// create the Stock Alert Rule panel
			this.fPrefs = this.attachMovie("MDialogBoxSymbol", "fPreferences", 1001, obj);
	
			// pass in the app name and LCService
			this.fPrefs.addItem({label: "Stock Alerts", linkage: "PrefsDialog", width: 400, height: 290, 
								sAppName: sAppName, oConn: oConn, oController:this.fPrefs});
			
			this.fPrefs.setTitle("Stock Watcher Preferences");
			this.fPrefs.setSelectedIndex(0);
			this.fPrefs.onClosed = onPrefsClosed;
		} else {
			this.fPrefs.doOpen();
		}

	}
	
	/**
	 * Called when the preferences dialog box has been closed
	 */
	function onPrefsClosed(Void):Void {

		StockUtils.trace("onPrefsClosed()", "StockWatcherApp");
		this.fPrefs.doClose();

		this.bIsPrefsOpen = false;
	}
	
	/**
	 * 
	 */
	function hidePreferences() {
		StockUtils.trace("hidePreferences()", "StockWatcherApp");
		
        fPrefs.hidePreferences();

        // if prefs are closing, then refresh the list
		refreshData();
    }

    // callback from the agent, when it pushes out a fresh date
	/**
	 * Called by the agent when a new round of data updates has been completed
	 */
    function setLastUpdate(when:Date):Void
    {
		StockUtils.trace("setLastUpdate(" + when + ")", "StockWatcherApp");
        displayDate(when);
		
		// hide status message and progress bar
		this.oShell.setStatus("");
		this.oShell.setProgress(0);
    }

	/**
	 * 
	 */
    function displayDate(when:Date):Void
    {
		if (when != undefined) {
			this.dLastUpdate = when;
			
			fUpdated.text = 
				"updated " + StockUtils.formatTime(this.dLastUpdate) + " " +
				StockUtils.formatDate(this.dLastUpdate);
		}
    }
	
	/**
	 * Called by the shell to get the smallest possible size of your application.
	 * @return Object An anonymous object with width and height properties.
	**/
	function getMinimumSize (Void):Object
    {
        return {width:500, height:400};
    }

    // end of central events - now for helper methods
    // initialization - set up the grid's columns
	/**
	 * 
	 */
    function setGridHeaders(grid:mx.controls.DataGrid):Void
    {
		StockUtils.trace("setGridHeaders()", "StockWatcherApp");
        // set grid headers
		grid.removeAllColumns();
        grid.columnNames = ["symbol", "company", "lastPrice", "change", "changePercent", "volume"];
	
        var width:Number = grid._width;
        var even:Number = Math.round(width/6);

        // space the grids evenly
        var c:mx.controls.gridclasses.DataGridColumn;
        c = grid.getColumnAt(0);
        c.headerText = "Symbol";
        c.width = Math.round(even * 0.5);

        c = grid.getColumnAt(1);
        c.headerText = "Company";
        c.width = Math.round(even * 1.5);
        
        c = grid.getColumnAt(2);
        c.headerText = "Last";
        c.width = even;

        c = grid.getColumnAt(3);
        c.headerText = "Change";
        c.width = even;
		
		c = grid.getColumnAt(4);
        c.headerText = "Change %";
        c.width = even;

        c = grid.getColumnAt(5);
        c.headerText = "Volume";
        c.width = even;
    }

    // process an item by selecting it, which should cause the company
    // information to display in the lower pane. If the company
    // doesn't already exist, add it.
	/**
	 * 
	 */
    function processSelectedItem(item:Object):Void
    {
        if (item.stock.symbol) {
            
            // if we already have this symbol, select it, otherwise search for it
            if (!findCompanyInGrid(item.stock.symbol)) {
                // place the received symbol in the search field and add it
                this.fSymbol.text = item.stock.symbol;
                this.fAddButton.onRelease();
            }
        }
    }

    //-------------------------------------------------------------------
    // View Updating
    //

    //
    // update the details pane by filling in all the data from the
    // item. If the item is empty, then we want to clear out the whole
    // details pane, setting each text field to ""
	/**
	 * 
	 */
    function showCompany(item:StockItem):Void
    {
		//StockUtils.traceProps("item=",item);
		with (this.fDetails) {
			if (!item) {
				// no item, just clear out the data
				fRemoveButton._visible = false;
				
				fTicker.text = "";
				fName.text = "";
				fLastTrade.text  = "";
				fLastTrade.text  = "";
				fChange.text  = "";
				fEPS.text  = "";
				fPE.text  = "";
				fMarketCap.text= "";
				fDivShare.text  = "";
				fOpen.text  = "";
				fYesterdayClose.text  = "";
				fBidAsk.text = "";
				fDayRange.text  = "";
				fYearRange.text  = "";
				fVolume.text  = "";
				fAverageVolume.text = "";
			} 
			else 
			{
				fRemoveButton._visible = true;
				
				// update details view
				fTicker.text = item.symbol;
				fName.text = item.company;
				
				fLastTrade.text = item.lastPrice;
				//fTradedDate.text = tradedDate+" "+tradedTime;
				fChange.text = item.change + " (" + item.changePercent + ")";
				fEPS.text = item.earnPerShare;
				fPE.text = item.PE;
				fMarketCap.text = item.marketCap;
				fDivShare.text = item.divShare;
	
				fOpen.text = item.open;
				fYesterdayClose.text = item.yesterdayClose;
				fBidAsk.text = item.bid + " - " + item.ask;
			
				fDayRange.text = item.dayLow + " - " + item.dayHigh;
				fYearRange.text = item.yearLow + " - " + item.yearHigh;
	
				fVolume.text = StockUtils.addCommas(item.volume);
				fAverageVolume.text = StockUtils.addCommas(item.averageVolume);	
			}        
		}
        updateBlastItem(item);
    }

	/**
	 * Selects a company in the grid
	 */
    function findCompanyInGrid(symbol:String):Boolean {
		StockUtils.trace("findCompanyInGrid(" + symbol + ")", "StockWatcherApp");
		this.fResultsGrid.selectedItem = undefined;
		this.fResultsGrid.selectedIndex = undefined;
		
        var len:Number = this.fResultsGrid.length;
		var nIndex:Number = 0;
		var bFound:Boolean = false;
		
        for (var i:Number=0; i<len; i++) {
            if (fResultsGrid.getItemAt(i).symbol == symbol) {
				StockUtils.trace("...found at index " + i, "StockWatcherApp");				
                nIndex = i;
				bFound = true;
				break;
            }
        }
		this.fResultsGrid.selectedIndex = nIndex;
		showSelectedCompany();
        return bFound;
    }

	/**
	 * Updates the currently selected item to enable the Blast menu
	 */
    function updateBlastItem(item:Object):Void
    {
        // create a new object to describe the current selection
        var metadata:Array = new Array();
        var stockItem:SelectedItem =
            new SelectedItem("http://download.macromedia.com/pub/central/schemas#", "stock");
	
        // add description of this stock
        stockItem.stock = item;
        stockItem.stock.date = StockUtils.formatDate(dLastUpdate) + " " +
            StockUtils.formatTime(dLastUpdate);
	
        // push it onto the selected item array
        metadata.push(stockItem);

        // tell Central about this selection
        this.oShell.setSelectedItem(metadata, item.symbol+" selected");
    }

	/**
	 * oConn.getLastUpdate is synchronous, so this is the result of the call
	 */
    function getLastUpdate_Result(result)
    {
		StockUtils.trace("getLastUpdate_Result(" + result + ")", "StockWatcherApp");
        displayDate(result);
		setGridHeaders(this.fResultsGrid);
		refreshGrid();
    }
	
	/* POD MANAGEMENT */
	
	/**
	 * 
	 */
	function openStockPod()
	{
		StockUtils.trace("openStockPod()", "StockWatcherApp");
		viewPod("Stock Watcher");
	}
	
	/**
	 * Open a named pod if it's not already open
	 */
	public function viewPod(podName:String):Void
	{
		// check if it's already viewed, because we don't want to open a second one
		if (!isPodViewed(podName)) {
			this.oShell.viewPod(this.getPodID(podName));
		} 
	}
	
	/**
	 * 
	 */
	public function removePod(podName:String):Void
	{
		this.oShell.removePod(this.getPodID(podName));
	}
	
	/**
	 * 
	 */
	public function getPodID(podName:String):Number
	{
		var aPods:Array = oShell.getPods();
		var podId:Number = -1;
		for( var i in aPods)
		{
			if( aPods[i].name == podName )
			{
				podId = aPods[i].id;
				break;
			}
		}
		return podId;
	}
	
	/** 
     * 
     */
	public function isPodCreated(podName:String):Boolean
	{
		var aPods:Array = this.oShell.getPods();
		for( var i in aPods)
		{
			if( aPods[i].name == podName )
			{
				return true;
			}
		}
		return false;
	}
	
	/**
     * 
     */
	public function isPodViewed(podName:String, symbol:String) : Boolean
	{
		var aPods : Array = this.oShell.getViewedPods();
		for( var i in aPods)
		{
			if( aPods[i].podData.name == podName)
			{
				return true;
			}
		}
		return false;
	}
	
	/**
     * 
     */
	public function isPodCollapsed(podName:String):Boolean
	{
		var aPods:Array = this.oShell.getViewedPods();
		for(var i in aPods)
		{
			if( aPods[i].podData.name == podName )
			{
				return aPods[i].podData.collapsed;
				break;
			}
		}
	}
	
	/**
	 * 
	 */
	function showError(sMsg:String) {
		mx.controls.Alert.show(sMsg, "An Error Occurred", Alert.OK,this, this.onAlertClosed);
	}
	
	/**
	 * 
	 */
	function onAlertClosed() {
		// not implemented
	}
}
